from django.db.models import Count
from django_tables2.export.export import TableExport
from netbox.views import generic
from ipam.models import VLAN
from .models import VLANGroupSet
from .forms import VLANGroupSetForm
from .tables import VLANGroupSetVLANTable, VLANGroupSetTable


class VLANGroupSetView(generic.ObjectView):
    queryset = VLANGroupSet.objects.all()

    def get_extra_context(self, request, instance):
        show_available = bool(request.GET.get('show_available', 'true') == 'true')
        show_assigned = bool(request.GET.get('show_assigned', 'true') == 'true')
        vlan_groups = instance.vlan_groups.all()
        vlan_group_vlans = instance.vlans
        requested_vlans = [
            x for x in vlan_group_vlans
            if show_available and x['status'] == 'Available' or show_assigned and x['status'] == 'Assigned'
        ]
        vlans_table = VLANGroupSetVLANTable(requested_vlans, vlan_groups=vlan_groups)
        vlans_table.configure(request)

        return {
            'vlans_table': vlans_table,
            'show_available': show_available,
            'show_assigned': show_assigned,
        }


class VLANGroupSetListView(generic.ObjectListView):
    queryset = VLANGroupSet.objects.annotate(
        vlan_group_count=Count('vlan_groups')
    )
    table = VLANGroupSetTable


class VLANGroupSetEditView(generic.ObjectEditView):
    queryset = VLANGroupSet.objects.all()
    form = VLANGroupSetForm


class VLANGroupSetDeleteView(generic.ObjectDeleteView):
    queryset = VLANGroupSet.objects.all()


class VLANGroupSetExportVLANs(generic.ObjectView):
    queryset = VLANGroupSet.objects.all()

    def get(self, request, **kwargs):
        instance = self.get_object(**kwargs)
        vlan_groups = instance.vlan_groups.all()
        vlan_group_vlans = instance.vlans
        vlans_table = VLANGroupSetVLANTable(
            vlan_group_vlans, vlan_groups=vlan_groups)
        vlans_table.configure(request)

        vlans_table = VLANGroupSetVLANTable(
            vlan_group_vlans, vlan_groups=vlan_groups)
        vlans_table.configure(request)
        exporter = TableExport('csv', vlans_table)
        return exporter.response(f'VLANGroupSetVLANs_{instance.name}.csv')



from netbox.models import NetBoxModel
from django.urls import reverse
import django_tables2 as tables
from netbox.tables import NetBoxTable, BaseTable
from .models import VLANGroupSet

VLAN_STATUS = """
{% if record.status == 'Available' %}
    <span class="badge bg-success">{{ record.status }}</span>
{% else %}
    <span class="badge bg-danger">{{ record.status }}</span>
{% endif %}
"""

VLAN_DATA = """
{% load view_helpers %}
{% with vlan=record|get_vlan_by_group:vlan_group %}
    {% if vlan %}
        <a href="{{ vlan.get_absolute_url }}">
            {{ vlan.name }}
        </a>
    {% else %}
        <a href="{% url 'ipam:vlan_add' %}?vid={{ vlan.vid }}&group={{ vlan_group.id }}" class="btn btn-sm btn-success">
            Add VLAN
        </a>
    {% endif %}
{% endwith %}
"""


class VLANGroupSetTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    vlan_group_count = tables.Column()

    class Meta(NetBoxTable.Meta):
        model = VLANGroupSet
        fields = (
            'pk',
            'id',
            'name',
            'vlan_group_count',
            'description',
            'comments'
        )
        default_columns = (
            'name',
            'vlan_group_count',
            'description'
        )


class VLANGroupSetVLANTable(BaseTable):
    vid = tables.Column(
        verbose_name='VID',
        linkify=lambda record: f'{reverse("ipam:vlan_list")}?vid={record["vid"]}'
    )
    status = tables.TemplateColumn(template_code=VLAN_STATUS)

    class Meta(BaseTable.Meta):
        model = NetBoxModel
        template_name = 'netbox_vlan_manager/vlangroupset_vlans.html'
        empty_text = 'No VLANs found'
        fields = (
            'vid',
            'status',
        )
        row_attrs = {
            'class': lambda record: 'success' if record['status'] == 'Available' else '',
        }

    def __init__(self, *args, vlan_groups=None, **kwargs):
        if vlan_groups is None:
            vlan_groups = []

        self.Meta.fields = list(self.Meta.fields)
        extra_columns = []
        for vlan_group in vlan_groups:
            column_name = f'vlangroup_{vlan_group.id}'
            column = (
                column_name,
                tables.TemplateColumn(
                    verbose_name=f'{vlan_group.name}',
                    template_code=VLAN_DATA,
                    extra_context={'vlan_group': vlan_group},
                )
            )
            extra_columns.append(column)
            # TODO: There's probably a more clever way to accomplish this
            if column_name not in self.Meta.fields:
                self.Meta.fields.append(column_name)

        super().__init__(*args, extra_columns=extra_columns, **kwargs)



from django.urls import path
from netbox.views.generic import ObjectChangeLogView
from . import models, views

urlpatterns = (
    path(
        'vlan-group-sets/',
        views.VLANGroupSetListView.as_view(),
        name='vlangroupset_list'
    ),
    path(
        'vlan-group-sets/add/',
        views.VLANGroupSetEditView.as_view(),
        name='vlangroupset_add'
    ),
    path(
        'vlan-group-sets/<int:pk>/',
        views.VLANGroupSetView.as_view(),
        name='vlangroupset'
    ),
    path(
        'vlan-group-sets/<int:pk>/edit/',
        views.VLANGroupSetEditView.as_view(),
        name='vlangroupset_edit'
    ),
    path(
        'vlan-group-sets/<int:pk>/delete/',
        views.VLANGroupSetDeleteView.as_view(),
        name='vlangroupset_delete'
    ),
    path(
        'vlan-group-sets/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='vlangroupset_changelog',
        kwargs={
            'model': models.VLANGroupSet
        }
    ),
    path(
        'vlan-group-sets/<int:pk>/export-vlans/',
        views.VLANGroupSetExportVLANs.as_view(),
        name='export_vlans'
    ),
)



# Generated by Django 4.1.6 on 2023-02-18 19:41

from django.db import migrations, models
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('extras', '0084_staging'),
        ('ipam', '0063_standardize_description_comments'),
    ]

    operations = [
        migrations.CreateModel(
            name='VLANGroupSet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vlan_groups', models.ManyToManyField(blank=True, related_name='vlan_group_sets', to='ipam.vlangroup')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
    ]



import codecs
import os.path

from setuptools import find_packages, setup


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


long_description = read("README.md")

setup(
    name='netbox-vlan-manager',
    version=get_version('netbox_vlan_manager/version.py'),
    description='VLAN Manager for multiple VLAN Groups',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/miyuk/netbox-vlan-manager/",
    author='miyuk',
    author_email='miyuk@miyuk.net',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ]
)
